package com.ruoyi.tc.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.common.core.domain.TreeSelect;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.tc.entity.AssetTask;
import com.ruoyi.tc.entity.Unit;
import com.ruoyi.tc.entity.request.GeneralQueryRequest;
import com.ruoyi.tc.entity.request.UnitRequest;
import com.ruoyi.tc.entity.response.AssetSysUserExport;

import java.util.List;

/**
 * 单位信息（unit_info）表业务层
 *
 * @author du
 * @since 2024/11/13 15:02
 */
public interface UnitService extends IService<Unit> {

    /**
     * 分页条件查询所有单位列表
     *
     * @param unit 用户信息
     * @return 用户信息集合信息
     */
    Page<Unit> page(Page<Unit> page, UnitRequest unit);

    /**
     * 根据条件查询所有单位列表
     *
     * @param unit 用户信息
     * @return 用户信息集合信息
     */
    List<Unit> selectUnitList(UnitRequest unit);

    /**
     * 根据id获取详细信息
     */
    Unit getById(Long id);

    /**
     * 检查用户表是否存在该用户
     */
    String validUser(Unit unit);

    /**
     * 根据id列表查询单位
     */
    String selectByIds(Long id);

    /**
     * 逻辑删除用户
     */
    void deleteUsers(String userNames);

    /**
     * 逻辑删除单位
     */
    void deleteUnits(Long ids);

    /**
     * 分页条件查询单位列表过滤
     *
     * @param unit 用户信息
     * @return 用户信息集合信息
     */
    Page<Unit> dwList(Page<Unit> page, UnitRequest unit);

    /**
     * 根据查询条件查询所有单位列表
     *
     * @param req 查询条件
     * @return 单位
     */
    List<Unit> selectAllUnit(GeneralQueryRequest req);

    /**
     * 根据单位名称和5种资产类型查询
     */
    Integer selectTaskId(String dwmc,String type);

    /**
     * 根据单位名称查询
     */
    List<AssetTask> selectTaskIdToDwmc(String dwmc);


    /**
     * 根据单位查询资产总数
     *
     * @param part 单位
     * @return
     */
    Integer findBydwmc(String part);


    /**
     * 根据资产类型返回组织架构
     */
    List<TreeSelect> getSchema(String type);

    /**
     * 查询所有单位
     */
    List<AssetSysUserExport> exportAllPassword();


    /**
     * 单位生成随机密码
     */
    int updatePasswordUser(AssetSysUserExport x);
}
